# Discarding Invalid Optional Fields

In the OpenRTB Ecto library, we have implemented functionality to gracefully handle invalid data in optional fields. Rather than rejecting an entire request because of a single invalid optional field, the library now discards only the invalid fields and accepts the rest of the valid data.

## How It Works

1. The `SafeSchema` module provides two primary functions:
   - `safe_cast/3`: A replacement for `cast/3` that discards invalid fields
   - `safe_cast_embed/3`: A replacement for `cast_embed/3` that discards invalid embedded schemas

2. When a schema is defined with `use OpenRtbEcto.SafeSchema`, these functions become available within the module.

3. The `OpenRtbEcto.cast/2` function has been updated to check for errors only on required fields, applying changes even if there are errors on optional fields.

## Using This Functionality

To use this functionality in your schemas:

```elixir
defmodule YourSchema do
  use OpenRtbEcto.SafeSchema
  
  @primary_key false
  embedded_schema do
    field(:id)
    field(:some_optional_field, :integer)
    embeds_one(:some_optional_embed, OptionalEmbed)
  end
  
  def changeset(schema, attrs \\ %{}) do
    schema
    |> safe_cast(attrs, [:id, :some_optional_field])
    |> safe_cast_embed(:some_optional_embed)
    |> validate_required([:id])
  end
end
```

## Custom Validation

For custom validation that should discard invalid data instead of marking the changeset as invalid:

```elixir
def validate_some_rule(changeset) do
  case get_change(changeset, :some_field) do
    nil -> changeset
    value when is_valid(value) -> changeset
    _ -> delete_change(changeset, :some_field)
  end
end
```

## Examples

### Optional Fields with Invalid Values

```elixir
data = %{
  id: "valid-id",
  optional_integer: "not-an-integer" # invalid
}

# Instead of rejecting the entire request, only the invalid field is discarded
{:ok, result} = OpenRtbEcto.cast(YourSchema, data)
# result.id == "valid-id"
# result.optional_integer == nil
```

### Invalid Optional Embedded Schemas

```elixir
data = %{
  id: "valid-id",
  optional_embed: %{
    required_field: nil # missing required field
  }
}

# Instead of rejecting the entire request, the invalid embed is discarded
{:ok, result} = OpenRtbEcto.cast(YourSchema, data)
# result.id == "valid-id"
# result.optional_embed == nil
```